import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';

// Define the initial state using that type
export const initialState: {
  queryListOpen: boolean;
  queryStatusList: {
    queries: Record<string, any>;
    queryIDsOrder: string[];
  };
  functionsMenuOpen: boolean;
  currentDatabaseKey: string;
  elementsperDatabaseObject: Record<string, number>;
  autoSendQueries: boolean;
  errors: string[];
} = {
  queryListOpen: false,
  queryStatusList: { queries: {}, queryIDsOrder: [] },
  functionsMenuOpen: false,
  currentDatabaseKey: '',
  elementsperDatabaseObject: {},
  autoSendQueries: true,
  errors: [],
};

export const configSlice = createSlice({
  name: 'config',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    addError: (state, action: PayloadAction<string>) => {
      console.error('Error Received!', action.payload);
      state.errors.push(action.payload);
    },
    removeLastError: (state) => {
      state.errors.shift();
    },
  },
});

export const { addError, removeLastError } = configSlice.actions;

// Other code such as selectors can use the imported `RootState` type
export const configState = (state: RootState) => state.config;

export default configSlice.reducer;
