/**
 * This program has been developed by a student from the bachelor Information Science at
 * Utrecht University within the Research Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import { ViewModel, Command, Edge, Graph, Coordinates } from './types';
import { calcNodeDistance } from './utils';
import * as d3 from 'd3';

export const fanEdges = (graph: Graph, selectedNode: string | null, maxWidth: number) => {
  const edgeGroups: { [key: string]: Edge[] } = {};

  graph.data.edges.forEach((edge: Edge) => {
    const key: string = `${edge.from}-${edge.to}`;
    if (!edgeGroups[key]) {
      edgeGroups[key] = [edge];
    } else {
      edgeGroups[key].push(edge);
    }
  });

  const fanned: Edge[] = [];

  const colorScale: d3.ScaleLinear<string, string> = d3.scaleLinear<string>().domain([0, 2000]).range(['yellow', 'red']);

  Object.values(edgeGroups).forEach((bundle) => {
    let adjacentNodeSelected: boolean = false;
    if ((selectedNode !== null && selectedNode === bundle[0].from) || selectedNode === bundle[0].to) {
      adjacentNodeSelected = true;
    }

    if (selectedNode && !adjacentNodeSelected) {
      return;
    }

    const prediction: number = bundle[0].attributes.prediction;
    const color: string = colorScale(prediction);
    const rgb: string[] = color.replace(/[^\d,]/g, '').split(',');
    const num_edges: number = bundle.length;

    if (bundle.length === 1) {
      fanned.push({
        ...bundle[0],
        from_coordinates: bundle[0].from_coordinates,
        to_coordinates: bundle[0].to_coordinates,
        color: rgb.map(Number),
        numEdges: num_edges,
      });
    } else {
      const source: Coordinates = bundle[0].from_coordinates;
      const target: Coordinates = bundle[0].to_coordinates;
      const node_distance: number = calcNodeDistance(source, target);
      const inViewport: boolean = node_distance < maxWidth - 20;

      bundle.forEach((edge: Edge, index: number) => {
        const newEdge: Edge = {
          ...edge,
          from_coordinates: source,
          to_coordinates: target,
          color: rgb.map(Number),
          numEdges: num_edges,
        };

        if (inViewport) {
          fanned.push(newEdge);
        } else {
          fanned.push({
            ...newEdge,
            tilt: (index / num_edges) * Math.PI,
            numEdges: 1,
          });
        }
      });
    }
  });

  return fanned;
};
